@@ -39,10 +39,12 @@ class ClassTemplateDecl;
 class ClassTemplateSpecializationDecl;
 class Decl;
 class DeclContext;
+class DeclRefExpr;
 class Expr;
 class FunctionDecl;
 class NamedDecl;
 class NamespaceDecl;
+class Sema;
 class Stmt;
 class TagDecl;
 class TemplateDecl;
@@ -50,6 +52,8 @@ class TemplateName;
 class TranslationUnitDecl;
 class TypeDecl;
 class ValueDecl;
+class VarDecl;
+class VarTemplateSpecializationDecl;
 }  // namespace clang
 
 namespace llvm {
@@ -533,6 +537,11 @@ internal::DynCastPtr<T> DynCastFrom(T* ptr) {
 // Returns true if the decl is for a templatized function.
 bool IsTemplatizedFunctionDecl(const clang::FunctionDecl* decl);
 
+// Returns true if the given declaration is an implicitly instantiated variable
+// template specialization, as opposed to explicit instantiations or full
+// specializations.
+bool IsImplicitInstantiation(const clang::VarDecl*);
+
 // Returns true if the given class has at least one implicit
 // conversion constructor.
 bool HasImplicitConversionCtor(const clang::CXXRecordDecl* cxx_class);
@@ -598,6 +607,11 @@ TemplateInstantiationData GetTplInstDataForFunction(
     const clang::FunctionDecl* decl, const clang::Expr* calling_expr,
     std::function<set<const clang::Type*>(const clang::Type*)> provided_getter);
 
+TemplateInstantiationData GetTplInstDataForVariable(
+    const clang::VarTemplateSpecializationDecl*,
+    const clang::DeclRefExpr*,
+    std::function<set<const clang::Type*>(const clang::Type*)> provided_getter);
+
 // If class_decl is instantiated from a class template,
 // returns the decl for that template; otherwise returns class_decl.
 // As an example, consider this code:
@@ -781,7 +795,7 @@ bool InvolvesTypeForWhich(const clang::Type* type,
 bool IsPointerOrReferenceAsWritten(const clang::Type* type);
 
 // This function removes a reference even it is hidden by some type sugar.
-const clang::Type* RemoveReference(const clang::Type*);
+clang::QualType RemoveReference(clang::QualType);
 
 // This function doesn't "see through" type sugar.
 const clang::Type* RemoveReferenceAsWritten(const clang::Type* type);
@@ -878,11 +892,36 @@ std::vector<const clang::Type*> GetCanonicalArgComponents(
 
 bool IsReferenceToModifiableLValue(const clang::Type*);
 
+// Returns true if the given type is a reference that can bind to a temporary
+// object, i.e. when it is either an rvalue reference or an lvalue reference to
+// a non-volatile const type (C++17 [dcl.init.ref]p.5.2).
+bool RefCanBindToTemp(const clang::Type*);
+
 // Returns true when the arguments are pointer types referring to classes or
 // structs related by direct inheritance. Also checks that their qualifiers
 // allow the conversion.
-bool IsDerivedToBasePtrConvertible(const clang::Type* derived_ptr_type,
-                                   const clang::Type* base_ptr_type);
+bool IsDerivedToBasePtrConvertible(clang::QualType derived_ptr_type,
+                                   clang::QualType base_ptr_type);
+
+// Pointers to members of a class can be implicitly converted to pointers
+// to members of its derived class (looks like the opposite to the conventional
+// derived-to-base pointer conversion) when the member types are identical or
+// related by certain standard conversions, like in this case:
+// int Base::* p1;
+// const int Derived::* p2 = p1;
+// This function checks if it actually can be done. If either of the given types
+// is not a MemberPointerType, returns false.
+bool IsBaseToDerivedMemPtrConvertible(const clang::Type* base_mem_ptr_type,
+                                      const clang::Type* derived_mem_ptr_type,
+                                      clang::Sema&);
+
+// Checks whether an object of the type 'to' may be copy-initialized from
+// an object of the type 'from'. 'conv_loc' must be valid, otherwise clang may
+// crash on instantiation of a class template specialization for 'from' type.
+bool IsConvertible(clang::QualType from,
+                   clang::QualType to,
+                   clang::SourceLocation conv_loc,
+                   clang::Sema&);
 
 // --- Utilities for Stmt.
 
