@@ -10,32 +10,47 @@
 #include "iwyu_globals.h"
 
 #include <algorithm>                    // for sort, make_pair
+#include <climits>
 #include <cstdio>                       // for printf
 #include <cstdlib>                      // for atoi, exit, getenv
+#include <cstring>
 #include <map>                          // for map
 #include <set>                          // for set
 #include <string>                       // for string, operator<, etc
 #include <utility>                      // for make_pair, pair
 
+#include "clang/AST/PrettyPrinter.h"
+#include "clang/Basic/DirectoryEntry.h"
+#include "clang/Basic/LangOptions.h"
+#include "clang/Basic/Version.h"
+#include "clang/Driver/ToolChain.h"
+#include "clang/Frontend/CompilerInstance.h"
+#include "clang/Lex/DirectoryLookup.h"
+#include "clang/Lex/HeaderSearch.h"
+#include "clang/Lex/Preprocessor.h"
 #include "iwyu_cache.h"
-#include "iwyu_include_picker.h"
 #include "iwyu_getopt.h"
+#include "iwyu_include_picker.h"
 #include "iwyu_lexer_utils.h"
 #include "iwyu_location_util.h"
 #include "iwyu_path_util.h"
 #include "iwyu_port.h"  // for CHECK_, etc
 #include "iwyu_regex.h"
-#include "iwyu_stl_util.h"
 #include "iwyu_string_util.h"
 #include "iwyu_verrs.h"
 #include "iwyu_version.h"
-#include "llvm/Support/raw_ostream.h"
-#include "clang/AST/PrettyPrinter.h"
-#include "clang/Basic/FileManager.h"
-#include "clang/Basic/Version.h"
-#include "clang/Lex/HeaderSearch.h"
+#include "llvm/ADT/StringRef.h"
+#include "llvm/Option/ArgList.h"
 
-using clang::DirectoryEntry;
+using clang::CompilerInstance;
+using clang::HeaderSearch;
+using clang::LangOptions;
+using clang::OptionalDirectoryEntryRef;
+using clang::OptionalFileEntryRef;
+using clang::PrintingPolicy;
+using clang::SourceManager;
+using clang::driver::ToolChain;
+using clang::getClangFullVersion;
 using std::make_pair;
 using std::map;
 using std::string;
@@ -44,10 +59,10 @@ using std::vector;
 namespace include_what_you_use {
 
 static CommandlineFlags* commandline_flags = nullptr;
-static clang::SourceManager* source_manager = nullptr;
+static SourceManager* source_manager = nullptr;
 static IncludePicker* include_picker = nullptr;
-static const clang::LangOptions default_lang_options;
-static const clang::PrintingPolicy default_print_policy(default_lang_options);
+static const LangOptions default_lang_options;
+static const PrintingPolicy default_print_policy(default_lang_options);
 static SourceManagerCharacterDataGetter* data_getter = nullptr;
 static FullUseCache* function_calls_full_use_cache = nullptr;
 static FullUseCache* class_members_full_use_cache = nullptr;
@@ -67,7 +82,8 @@ static void PrintHelp(const char* extra_msg) {
          "        This flag may be specified multiple times to specify\n"
          "        multiple glob patterns.\n"
          "   --mapping_file=<filename>: gives iwyu a mapping file.\n"
-         "   --no_default_mappings: do not add iwyu's default mappings.\n"
+         "   --no_internal_mappings: do not add iwyu's internal mappings.\n"
+         "   --export_mappings=<dirpath>: writes out all internal mappings.\n"
          "   --pch_in_code: mark the first include in a translation unit as a\n"
          "        precompiled header.  Use --pch_in_code to prevent IWYU from\n"
          "        removing necessary PCH includes.  Though Clang forces PCHs\n"
@@ -104,7 +120,8 @@ static void PrintHelp(const char* extra_msg) {
          "   --verbose=<level>: the higher the level, the more output.\n"
          "   --quoted_includes_first: when sorting includes, place quoted\n"
          "        ones first.\n"
-         "   --cxx17ns: suggests the more concise syntax introduced in C++17\n"
+         "   --cxx17ns: use C++17 nested namespaces when suggesting additions\n"
+         "        of forward declarations.\n"
          "   --error[=N]: exit with N (default: 1) for iwyu violations\n"
          "   --error_always[=N]: always exit with N (default: 1) (for use\n"
          "        with 'make -k')\n"
@@ -112,6 +129,9 @@ static void PrintHelp(const char* extra_msg) {
          "   --regex=<dialect>: use specified regex dialect in IWYU:\n"
          "          llvm:       fast and simple (default)\n"
          "          ecmascript: slower, but more feature-complete\n"
+         "   --experimental=flag[,flag...]: enable experimental features\n"
+         "          clang_mappings: use Clang canonical standard library\n"
+         "                          mappings instead of internal mappings\n"
          "\n"
          "In addition to IWYU-specific options you can specify the following\n"
          "options without -Xiwyu prefix:\n"
@@ -128,8 +148,7 @@ static void PrintVersion() {
   if (!iwyu_rev.empty()) {
     llvm::outs() << " (git:" << iwyu_rev << ")";
   }
-  llvm::outs() << " based on " << clang::getClangFullVersion()
-               << "\n";
+  llvm::outs() << " based on " << getClangFullVersion() << "\n";
 }
 
 static bool ParseIntegerOptarg(const char* optarg, int* res) {
@@ -190,7 +209,7 @@ OptionsParser::~OptionsParser() {
 CommandlineFlags::CommandlineFlags()
     : transitive_includes_only(false),
       verbose(getenv("IWYU_VERBOSE") ? atoi(getenv("IWYU_VERBOSE")) : 1),
-      no_default_mappings(false),
+      no_internal_mappings(false),
       max_line_length(80),
       prefix_header_include_policy(CommandlineFlags::kAdd),
       pch_in_code(false),
@@ -214,7 +233,8 @@ int CommandlineFlags::ParseArgv(int argc, char** argv) {
     {"transitive_includes_only", no_argument, nullptr, 't'},
     {"verbose", required_argument, nullptr, 'v'},
     {"mapping_file", required_argument, nullptr, 'm'},
-    {"no_default_mappings", no_argument, nullptr, 'n'},
+    {"no_internal_mappings", no_argument, nullptr, 'n'},
+    {"no_default_mappings", no_argument, nullptr, 'n'},  // deprecated
     {"prefix_header_includes", required_argument, nullptr, 'x'},
     {"pch_in_code", no_argument, nullptr, 'h'},
     {"max_line_length", required_argument, nullptr, 'l'},
@@ -228,6 +248,8 @@ int CommandlineFlags::ParseArgv(int argc, char** argv) {
     {"error_always", optional_argument, nullptr, 'a'},
     {"debug", required_argument, nullptr, 'd'},
     {"regex", required_argument, nullptr, 'r'},
+    {"experimental", required_argument, nullptr, 'p'},
+    {"export_mappings", required_argument, nullptr, 'E'},
     {nullptr, 0, nullptr, 0}
   };
   static const char shortopts[] = "v:c:m:d:nr";
@@ -238,7 +260,9 @@ int CommandlineFlags::ParseArgv(int argc, char** argv) {
       case 't': transitive_includes_only = true; break;
       case 'v': verbose = atoi(optarg); break;
       case 'm': mapping_files.push_back(optarg); break;
-      case 'n': no_default_mappings = true; break;
+      case 'n':
+        no_internal_mappings = true;
+        break;
       case 'o': no_comments = true; break;
       case 'u': update_comments = true; break;
       case 'i':
@@ -308,6 +332,26 @@ int CommandlineFlags::ParseArgv(int argc, char** argv) {
           exit(EXIT_FAILURE);
         }
         break;
+      case 'p': {
+        // Split argument on comma and save in global, ignoring empty elements.
+        vector<string> flags = Split(optarg, ",", 0);
+        exp_flags.insert(flags.begin(),
+                         std::remove(flags.begin(), flags.end(), string()));
+        // Print all effective flags for traceability.
+        for (const string& f : exp_flags) {
+          llvm::errs() << "Experimental flag enabled: '" << f << "'\n";
+        }
+        break;
+      }
+      case 'E': {
+        // Handle --export_mappings immediately. We already depend on
+        // iwyu_include_picker here, and we want to run the export before the
+        // Clang/IWYU driver starts making demands for required inputs, etc.
+        string output_dirpath(optarg);
+        ExportInternalMappings(output_dirpath);
+        exit(EXIT_SUCCESS);
+        break;
+      }
       case -1:
         return optind;  // means 'no more input'
       default:
@@ -324,6 +368,10 @@ bool CommandlineFlags::HasDebugFlag(const char* flag) const {
   return dbg_flags.find(string(flag)) != dbg_flags.end();
 }
 
+bool CommandlineFlags::HasExperimentalFlag(const char* flag) const {
+  return exp_flags.find(string(flag)) != exp_flags.end();
+}
+
 // Though option -v prints version too, it isn't intercepted because it also
 // provides other functionality like printing clang invocation, header search
 // paths.
@@ -386,18 +434,18 @@ static vector<HeaderSearchPath> NormalizeHeaderSearchPaths(
 // Asks clang what the search-paths are for include files, normalizes
 // them, and returns them in a vector.
 static vector<HeaderSearchPath> ComputeHeaderSearchPaths(
-    clang::HeaderSearch* header_search) {
+    HeaderSearch* header_search) {
   map<string, HeaderSearchPath::Type> search_path_map;
   for (auto it = header_search->system_dir_begin();
        it != header_search->system_dir_end(); ++it) {
-    if (const DirectoryEntry* entry = it->getDir()) {
+    if (OptionalDirectoryEntryRef entry = it->getDirRef()) {
       const string path = NormalizeDirPath(MakeAbsolutePath(entry->getName().str()));
       search_path_map[path] = HeaderSearchPath::kSystemPath;
     }
   }
   for (auto it = header_search->search_dir_begin();
        it != header_search->search_dir_end(); ++it) {
-    if (const DirectoryEntry* entry = it->getDir()) {
+    if (OptionalDirectoryEntryRef entry = it->getDirRef()) {
       // search_dir_begin()/end() includes both system and user paths.
       // If it's a system path, it's already in the map, so everything
       // new is a user path.  The insert only 'takes' for new entries.
@@ -408,22 +456,45 @@ static vector<HeaderSearchPath> ComputeHeaderSearchPaths(
   return NormalizeHeaderSearchPaths(search_path_map);
 }
 
-void InitGlobals(clang::SourceManager* sm, clang::HeaderSearch* header_search) {
-  CHECK_(sm && "InitGlobals() needs a non-nullptr SourceManager");
-  source_manager = sm;
+static CStdLib DeriveCStdLib() {
+  if (GlobalFlags().no_internal_mappings)
+    return CStdLib::None;
+  if (GlobalFlags().HasExperimentalFlag("clang_mappings"))
+    return CStdLib::ClangSymbols;
+  return CStdLib::Glibc;
+}
+
+static CXXStdLib DeriveCXXStdLib(const CompilerInstance& compiler,
+                                 const ToolChain& toolchain) {
+  if (GlobalFlags().no_internal_mappings || !compiler.getLangOpts().CPlusPlus)
+    return CXXStdLib::None;
+  if (GlobalFlags().HasExperimentalFlag("clang_mappings"))
+    return CXXStdLib::ClangSymbols;
+
+  // Get standard library requested for the compilation. ToolChain caches the
+  // already-parsed args, so pass in an empty arglist.
+  llvm::opt::InputArgList nullargs;
+  switch (toolchain.GetCXXStdlibType(nullargs)) {
+    case ToolChain::CXXStdlibType::CST_Libcxx:
+      return CXXStdLib::Libcxx;
+    case ToolChain::CXXStdlibType::CST_Libstdcxx:
+      return CXXStdLib::Libstdcxx;
+  }
+  CHECK_UNREACHABLE_("covered switch for CXXStdlibType above");
+}
+
+void InitGlobals(CompilerInstance& compiler, const ToolChain& toolchain) {
+  source_manager = &compiler.getSourceManager();
   data_getter = new SourceManagerCharacterDataGetter(*source_manager);
-  vector<HeaderSearchPath> search_paths =
-      ComputeHeaderSearchPaths(header_search);
+  vector<HeaderSearchPath> search_paths = ComputeHeaderSearchPaths(
+      &compiler.getPreprocessor().getHeaderSearchInfo());
   SetHeaderSearchPaths(search_paths);
-  CStdLib cstdlib = CStdLib::Glibc;
-  CXXStdLib cxxstdlib = CXXStdLib::Libstdcxx;
-  if (GlobalFlags().no_default_mappings) {
-    cstdlib = CStdLib::None;
-    cxxstdlib = CXXStdLib::None;
-  }
+
+  RegexDialect regex_dialect = GlobalFlags().regex_dialect;
+  CStdLib cstdlib = DeriveCStdLib();
+  CXXStdLib cxxstdlib = DeriveCXXStdLib(compiler, toolchain);
+  include_picker = new IncludePicker(regex_dialect, cstdlib, cxxstdlib);
 
-  include_picker =
-      new IncludePicker(GlobalFlags().regex_dialect, cstdlib, cxxstdlib);
   function_calls_full_use_cache = new FullUseCache;
   class_members_full_use_cache = new FullUseCache;
 
@@ -450,7 +521,7 @@ CommandlineFlags* MutableGlobalFlagsForTesting() {
   return commandline_flags;
 }
 
-clang::SourceManager* GlobalSourceManager() {
+SourceManager* GlobalSourceManager() {
   CHECK_(source_manager && "Must call InitGlobals() before calling this");
   return source_manager;
 }
@@ -465,7 +536,7 @@ IncludePicker* MutableGlobalIncludePicker() {
   return include_picker;
 }
 
-const clang::PrintingPolicy& DefaultPrintPolicy() {
+const PrintingPolicy& DefaultPrintPolicy() {
   return default_print_policy;
 }
 
@@ -487,7 +558,7 @@ void AddGlobToReportIWYUViolationsFor(const string& glob) {
   commandline_flags->check_also.insert(NormalizeFilePath(glob));
 }
 
-bool ShouldReportIWYUViolationsFor(const clang::FileEntry* file) {
+bool ShouldReportIWYUViolationsFor(OptionalFileEntryRef file) {
   const string filepath = GetFilePath(file);
   for (const string& glob : GlobalFlags().check_also)
     if (GlobMatchesPath(glob.c_str(), filepath.c_str()))
@@ -500,7 +571,7 @@ void AddGlobToKeepIncludes(const string& glob) {
   commandline_flags->keep.insert(NormalizeFilePath(glob));
 }
 
-bool ShouldKeepIncludeFor(const clang::FileEntry* file) {
+bool ShouldKeepIncludeFor(OptionalFileEntryRef file) {
   if (GlobalFlags().keep.empty())
     return false;
   const string filepath = GetFilePath(file);
@@ -518,9 +589,12 @@ void InitGlobalsAndFlagsForTesting() {
   data_getter = nullptr;
   CStdLib cstdlib = CStdLib::Glibc;
   CXXStdLib cxxstdlib = CXXStdLib::Libstdcxx;
-  if (GlobalFlags().no_default_mappings) {
+  if (GlobalFlags().no_internal_mappings) {
     cstdlib = CStdLib::None;
     cxxstdlib = CXXStdLib::None;
+  } else if (GlobalFlags().HasExperimentalFlag("clang_mappings")) {
+    cstdlib = CStdLib::ClangSymbols;
+    cxxstdlib = CXXStdLib::ClangSymbols;
   }
 
   include_picker =
