@@ -16,19 +16,24 @@
 #ifndef INCLUDE_WHAT_YOU_USE_IWYU_OUTPUT_H_
 #define INCLUDE_WHAT_YOU_USE_IWYU_OUTPUT_H_
 
+#include <cstddef>
 #include <map>                          // for map
 #include <set>                          // for set
 #include <string>                       // for string, operator<
 #include <vector>                       // for vector
 
+#include "clang/AST/Decl.h"
+#include "clang/Basic/FileEntry.h"
+#include "clang/Basic/SourceLocation.h"
 #include "iwyu_port.h"  // for CHECK_
 #include "iwyu_stl_util.h"
 #include "iwyu_use_flags.h"
-#include "clang/AST/Decl.h"
-#include "clang/Basic/SourceLocation.h"
+
+// TODO: Clean out pragmas as IWYU improves.
+// IWYU pragma: no_include "clang/Basic/CustomizableOptional.h"
 
 namespace clang {
-class FileEntry;
+class ElaboratedTypeLoc;
 class UsingDecl;
 }  // namespace clang
 
@@ -47,22 +52,24 @@ class OneUse {
  public:
   enum UseKind { kFullUse, kForwardDeclareUse };
 
+  // This constructor is used to track nominal decl uses.
   OneUse(const clang::NamedDecl* decl,
          clang::SourceLocation use_loc,
          clang::SourceLocation decl_loc,
          UseKind use_kind,
          UseFlags flags,
          const char* comment);
-  // Both dfn_file and dfn_filepath are specified to allow to create OneUse
-  // with dfn_filepath and without dfn_file.  For example, in
-  // IwyuBaseAstVisitor::VisitCXXNewExpr we make a guess that placement
-  // operator new is called (which is defined in <new>), but we don't have
-  // <new> FileEntry.
-  OneUse(const string& symbol_name,
-         const clang::FileEntry* dfn_file,
-         const string& dfn_filepath,
+
+  // This constructor is used to track macro uses and abstract symbol uses (such
+  // as placement operator new, which requires <new>).
+  OneUse(const string& symbol_name, clang::OptionalFileEntryRef dfn_file,
          clang::SourceLocation use_loc);
 
+  // This constructor is used to track include file uses. See
+  // ReportIncludeFileUse for details.
+  OneUse(clang::OptionalFileEntryRef included_file,
+         const string& quoted_include);
+
   const string& symbol_name() const {
     return symbol_name_;
   }
@@ -72,7 +79,7 @@ class OneUse {
   const clang::NamedDecl* decl() const {
     return decl_;
   }
-  const clang::FileEntry* decl_file() const {
+  clang::OptionalFileEntryRef decl_file() const {
     return decl_file_;
   }
   const string& decl_filepath() const {
@@ -129,7 +136,7 @@ class OneUse {
   string short_symbol_name_;       // 'short' form of the symbol being used
   const clang::NamedDecl* decl_;   // decl of the symbol, if we know it
   clang::SourceLocation decl_loc_;     // where the decl is attributed to live
-  const clang::FileEntry* decl_file_;  // file entry where the symbol lives
+  clang::OptionalFileEntryRef decl_file_;  // file entry where the symbol lives
   string decl_filepath_;           // filepath where the symbol lives
   clang::SourceLocation use_loc_;  // where the symbol is used from
   UseKind use_kind_;               // kFullUse or kForwardDeclareUse
@@ -144,7 +151,8 @@ class OneUse {
 class OneIncludeOrForwardDeclareLine {
  public:
   explicit OneIncludeOrForwardDeclareLine(const clang::NamedDecl* fwd_decl);
-  OneIncludeOrForwardDeclareLine(const clang::FileEntry* included_file,
+  explicit OneIncludeOrForwardDeclareLine(clang::ElaboratedTypeLoc);
+  OneIncludeOrForwardDeclareLine(clang::OptionalFileEntryRef included_file,
                                  const string& quoted_include, int linenum);
 
   const string& line() const {
@@ -158,6 +166,9 @@ class OneIncludeOrForwardDeclareLine {
   bool is_present() const {
     return is_present_;
   }
+  bool is_elaborated_type() const {
+    return is_elaborated_type_;
+  }
   const map<string, int>& symbol_counts() const {
     return symbol_counts_;
   }
@@ -168,7 +179,7 @@ class OneIncludeOrForwardDeclareLine {
     return quoted_include_;
   }
 
-  const clang::FileEntry* included_file() const {
+  clang::OptionalFileEntryRef included_file() const {
     CHECK_(IsIncludeLine() && "Must call included_file() on include lines");
     CHECK_(!fwd_decl_ && "included_file and fwd_decl are mutually exclusive");
     return included_file_;
@@ -203,17 +214,18 @@ class OneIncludeOrForwardDeclareLine {
   }
 
  private:
-  string line_;                     // '#include XXX' or 'class YYY;'
-  int start_linenum_;
-  int end_linenum_;
-  bool is_desired_;                 // IWYU will recommend this line
-  bool is_present_;                 // line was present before the IWYU run
-  map<string, int> symbol_counts_;  // how many times we referenced each symbol
+  string line_;  // '#include XXX' or 'class YYY;'
+  int start_linenum_ = -1;
+  int end_linenum_ = -1;
+  bool is_desired_ = false;          // IWYU will recommend this line
+  bool is_present_ = false;          // line was present before the IWYU run
+  bool is_elaborated_type_ = false;  // Fwd-decl introduced by elaborated type
+  map<string, int> symbol_counts_;   // how many times we referenced each symbol
   // Only either two following members are set for includes
-  string quoted_include_;           // quoted file name we're including
-  const clang::FileEntry* included_file_;  // the file we're including
+  string quoted_include_;  // quoted file name we're including
+  clang::OptionalFileEntryRef included_file_;  // the file we're including
   // ...or this member is set for the fwd-decl we're emitting.
-  const clang::NamedDecl* fwd_decl_;
+  const clang::NamedDecl* fwd_decl_ = nullptr;
 };
 
 // This class holds IWYU information about a single file (FileEntry)
@@ -226,7 +238,7 @@ class IwyuFileInfo {
  public:
   // TODO(csilvers): also take iwyufileinfos for 'associated' files (.h's).
   // And a source-manager.
-  IwyuFileInfo(const clang::FileEntry* this_file,
+  IwyuFileInfo(clang::OptionalFileEntryRef this_file,
                const IwyuPreprocessorInfo* preprocessor_info,
                const string& quoted_include_name);
 
@@ -240,6 +252,14 @@ class IwyuFileInfo {
   }
   void set_pch_in_code() { is_pch_in_code_ = true; }
 
+  clang::OptionalFileEntryRef file_entry() const {
+    return file_;
+  }
+
+  const set<string>& direct_includes() const {
+    return direct_includes_;
+  }
+
   // An 'associated' header is a header that this file #includes
   // (possibly indirectly) that we should treat as being logically
   // part of this file.  In particular, when computing the direct
@@ -252,12 +272,13 @@ class IwyuFileInfo {
   // Use these to register an iwyu declaration: either an #include,
   // a forward-declaration or a using-declaration.
 
-  void AddInclude(const clang::FileEntry* includee,
+  void AddInclude(clang::OptionalFileEntryRef includee,
                   const string& quoted_includee, int linenumber);
   // definitely_keep_fwd_decl tells us that we should never suggest
   // the fwd-decl be removed, even if we don't see any uses of it.
   void AddForwardDeclare(const clang::NamedDecl* fwd_decl,
                          bool definitely_keep_fwd_decl);
+  void AddElaboratedType(clang::ElaboratedTypeLoc);
 
   void AddUsingDecl(const clang::UsingDecl* using_decl);
 
@@ -274,7 +295,7 @@ class IwyuFileInfo {
   // only for placement operator new in templates (see
   // IwyuBaseAstVisitor::VisitCXXNewExpr).
   void ReportFullSymbolUse(clang::SourceLocation use_loc,
-                           const clang::FileEntry* dfn_file,
+                           clang::OptionalFileEntryRef dfn_file,
                            const string& symbol);
   // TODO(dsturtevant): Can we determine in_cxx_method_body? Do we care?
 
@@ -284,7 +305,7 @@ class IwyuFileInfo {
                       const string& symbol);
 
   // Called when somebody uses a macro defined in this file.
-  void ReportDefinedMacroUse(const clang::FileEntry* used_in);
+  void ReportDefinedMacroUse(clang::OptionalFileEntryRef used_in);
 
   // We only allow forward-declaring of decls, not arbitrary symbols.
   void ReportForwardDeclareUse(clang::SourceLocation use_loc,
@@ -300,16 +321,17 @@ class IwyuFileInfo {
   // This is used when we see a // NOLINT comment, for instance.  It says
   // '#include this header file as-is, without any public-header mapping.'
   // Input is the include-line as desired: '<string.h>' or '"ads/foo.h"'.
-  void ReportIncludeFileUse(const clang::FileEntry* included_file,
+  void ReportIncludeFileUse(clang::OptionalFileEntryRef included_file,
                             const string& quoted_include);
 
   // This is used when we see a file we want to keep not due to symbol-use
   // reasons.  For example, it can be #included with an "IWYU pragma: keep"
   // comment or it can be an x-macro.
-  void ReportKnownDesiredFile(const clang::FileEntry* included_file);
+  void ReportKnownDesiredFile(clang::OptionalFileEntryRef included_file);
 
   // This is used only in iwyu_preprocessor.cc.  TODO(csilvers): revamp?
-  const set<const clang::FileEntry*>& direct_includes_as_fileentries() const {
+  const set<clang::OptionalFileEntryRef>& direct_includes_as_fileentries()
+      const {
     return direct_includes_as_fileentries_;
   }
 
@@ -326,10 +348,6 @@ class IwyuFileInfo {
   size_t CalculateAndReportIwyuViolations();
 
  private:
-  const set<string>& direct_includes() const {
-    return direct_includes_;
-  }
-
   const set<string>& desired_includes() const {
     CHECK_(desired_includes_have_been_calculated_ &&
            "Must calculate desired includes before calling desired_includes()");
@@ -343,8 +361,8 @@ class IwyuFileInfo {
     return associated_quoted_includes;
   }
 
-  set<const clang::FileEntry*> AssociatedFileEntries() const {
-    set<const clang::FileEntry*> associated_file_entries;
+  set<clang::OptionalFileEntryRef> AssociatedFileEntries() const {
+    set<clang::OptionalFileEntryRef> associated_file_entries;
     for (const IwyuFileInfo* associated : associated_headers_)
       associated_file_entries.insert(associated->file_);
     return associated_file_entries;
@@ -365,7 +383,7 @@ class IwyuFileInfo {
   int EmitWarningMessages(const vector<OneUse>& uses);
 
   // The constructor arguments.  file_ is 'this file'.
-  const clang::FileEntry* file_;
+  clang::OptionalFileEntryRef file_;
   const IwyuPreprocessorInfo* preprocessor_info_;
 
   string quoted_file_;
@@ -397,21 +415,25 @@ class IwyuFileInfo {
   // We also hold the line information in a few other data structures,
   // for ease of references.
   set<string> direct_includes_;      // key is the quoted include, eg '<set>'
-  set<const clang::FileEntry*> direct_includes_as_fileentries_;
+  set<clang::OptionalFileEntryRef> direct_includes_as_fileentries_;
   set<const clang::NamedDecl*> direct_forward_declares_;
 
   // Holds files forced to be kept.  For example, files included with the
   // "IWYU pragma: keep" comment and x-macros.
-  set<const clang::FileEntry*> kept_includes_;
+  set<clang::OptionalFileEntryRef> kept_includes_;
 
   // Holds files using macros defined in this file.
-  set<const clang::FileEntry*> macro_users_;
+  set<clang::OptionalFileEntryRef> macro_users_;
 
   // What we will recommend the #includes to be.
   set<string> desired_includes_;
   bool desired_includes_have_been_calculated_;
 };
 
+// Returns true if a public header (i.e. with respect to mappings) for NamedDecl
+// is directly included.
+bool IsDirectlyIncluded(const clang::NamedDecl*, const IwyuFileInfo& includer);
+
 // Helpers for testing.
 
 namespace internal {
